#!/usr/bin/python

#Written by Dan Gerszewski
#March 2009

import os
import getopt, sys
import time
import fileinput

#import makeparticlerib

def usage():
	print "Script for rendering object files in pixie. "
	print "Files are named in a range starting from dump_1000000.obj.gz \n"  
	print "./render.py startindex endindex"
	print "Example: ./render.py 0 100 -p 8 -r myrib.rib -f large  "
	print "(will render dump_1000000.obj.gz through dump_1000100.obj.gz)\n"
	print "Options: "
	print "-r , --ribfile : specify the pixie rib file to copy into the current directory to use for rendering "
	print "-p , --processors : 1-8 specifiy the number of processors to render with"
	print "-s , --step : the step size to count between startindex and endindex"
	print "-h , --help : usage "
	print "-v , --verbose : verbose printing "
	print "-f , --format : large , small. Sets the image format: small = 640 360 1, large = 1280 720 1"


def print_timing(func):
	def wrapper(*arg):
		t1 = time.time()
		res = func(*arg)
		t2 = time.time()
		print '%s took %0.3f s' % (func.func_name, (t2-t1))
		return res
	return wrapper

#this is not used currently
@print_timing
def runpixie(command):
	if _verbose :
		print command;
	os.system(command);

def run(command):
	if _verbose :
		print command;
	os.system(command);


def main(argv):
	if len(argv) < 2:
		usage();
		sys.exit(2);

	i = int(argv[0]);
	lastindex = int(argv[1]);
	step = 1;
	processors = 0;
	format = ''
	ribfileToCopy = ''
	global _verbose
	_verbose = False;

	#the simulator runs the script like so ./pov/render.py so that the current
	#working dir is not pov so it won't find any obj files... so we change the dir to pov to fix this.
	if os.path.exists("pov"):
		os.chdir("pov");

	if len(argv) > 2:
		#get all the arguments
		try:
			opts, args = getopt.getopt(argv[2:], "hvr:p:s:f:", ["help", "verbose", "ribfile=", "processors=", "step=", "format="])
		except getopt.GetoptError:
			usage()
			sys.exit(2)
		for opt, arg in opts:
			if opt in ("-h", "--help"):
				usage()
				sys.exit()
			elif opt in ("-v", "--verbose"):
				_verbose = True
				print "Verbose On"
			elif opt in ("-s", "--step"):
				step = int(arg)
				print "Stepsize arg = " + str(step)
			elif opt in ("-p", "--processors"):
				processors = arg;
				print "Processors arg = " + arg
				#Option "limits" "int numthreads" [8]
			elif opt in ("-r", "--ribfile"):
				ribfileToCopy = arg;
				print "Using rib file %s " % ribfileToCopy
			elif opt in ("-f", "--format"):
				if arg.upper() == "SMALL" :
					#we write them in this order so that the correct format is last
					format = ['Format 1280 720 1', 'Format 640 360 1']
					print "Image format arg = small = 640 360"
				elif arg.upper() == "LARGE" :
					#write in this order since pixie uses the last one it reads in
					format = ['Format 640 360 1', 'Format 1280 720 1']
					print "Image format arg = large = 1280 720"
	#if a rib file is set with -r make sure it is valid and copy it to current dir for rendering
	#we don't want to overwrite our template rib file 
	if ribfileToCopy != '':			
		if not os.path.exists(ribfileToCopy):
			print "ERROR -- Ribfile %s is not found. Exiting --" % ribfileToCopy
			sys.exit(2)
		else:
			#back up the current Fluid.rib file just in case.. uncomment if needed
			#if os.path.exists("Fluid.rib"):
			#	run("mv Fluid.rib Fluid.rib.bak");
			run("cp "+ribfileToCopy+" Fluid.rib");	
	else:
		if not os.path.exists("Fluid.rib"):
			print "ERROR -- Fluid.rib not found in current dir. You can specify one to copy with -r "
			sys.exit(2)

	ribfile = "Fluid.rib" #name used for rib file... 

	if processors > 0 or format != '':
		#change the number of processors or Format in the rib file
		firsttime = True
		for line in fileinput.input(ribfile, inplace=True,backup='.rbak'):
			
			if processors > 0 and line.find('Option "limits" "int numthreads"') != -1:
				print 'Option "limits" "int numthreads" ['+str(processors)+']'
				continue
			#check if an arg set the format	
			if format != '': 
				#if a line contains Format then we remove it from the file(done by calling continue)
				if line.find('Format') != -1:
					#the first Format we find we print out both in the correct order.. the later formats
					#will be removed by the continue( print statements go to the file )
					if firsttime:
						print format[0].rstrip("\n");
						print format[1].rstrip("\n");
						firsttime = False
					#this continue will cause all the lines that contain Format 
					#to be excluded from the file.
					continue	
					
					
			print line.rstrip("\n"); #print adds a newline but line already has one
		fileinput.close()

		
	#print rib info
	print "\n--Rendering Parameters--"
	ribf = open(ribfile, 'r');
	riblines = ribf.readlines();
	format = ''
	for line in riblines:
		if line.find('Option "limits" "int numthreads"') != -1:
			print "Number of Processors = " + line[-3:-2]
		if line.find('Format') == 0:
			format = line
			
	print format.rstrip("\n");
	ribf.close();
	
	totalframes = (lastindex - i)/step + 1
	framecount = 0

	while(i <= lastindex):
		j = 1000000 + i;  #get frame number

		##TODO: add options for rendering the particle files and doing surface tiles
		####os.system("python makeparticlerib.py dparts_"+j+"

		t1 = time.time();
		#file names
		fluidrib = 'fluid.'+str(j)+'.rib';
		dumpfilegz = 'dump_'+str(j)+'.obj.gz';
		dumpfile = 'dump_'+str(j)+'.obj';
		if( os.path.exists(dumpfilegz) ):
			run('gunzip -c dump_'+str(j)+'.obj.gz > dump_'+str(j)+'.obj');
			
		run('~adamb/Code/Projects/BFAST/slcConvert '+dumpfile+' foo.'+str(j)+'.rib');
		t2 = time.time();
		run("cat "+ribfile+" | sed 's/FRAME/"+str(j)+"/g' > "+fluidrib);	
		run("rndr "+fluidrib);
		run("convert fluid."+str(j)+".tif fluid."+str(j)+".png");
				
		run("rm "+fluidrib);
		run("rm foo."+str(j)+".rib");
		run("rm fluid."+str(j)+".tif");
		
		if(os.path.exists(dumpfilegz)):
			run("rm dump_"+str(j)+".obj");
		t3 = time.time();
		framecount = framecount + 1
		print "(%d/%d)frame %d | slcConvert %0.3f s | rndr %0.3f s | %0.3f s total" % (framecount,totalframes,j,(t2-t1),(t3-t2),(t3-t1)) 
		i = i + step;








if __name__ == "__main__":
    main(sys.argv[1:])


