//-------------------------------------------------------------------
//-------------------------------------------------------------------
//
// LevelSet Surface Tracker
//   -- Code File
//  
// Primary Author: Adam Bargteil (adamb@cs.utah.edu)
// 
//-------------------------------------------------------------------
//-------------------------------------------------------------------
//
// Copyright (c) 2008-2009, Utah State Board of Regents.  All
// rights reserved.
//
// Redistribution and use in source and binary forms, with or without
// modification, are permitted provided that the following conditions are
// met:
//
//   Redistributions of source code must retain the above copyright
//  notice, this list of conditions and the following disclaimer.
//
//  Redistributions in binary form must reproduce the above copyright
//  notice, this list of conditions and the following disclaimer in the
//  documentation and/or other materials provided with the distribution.
//
//  Redistributions in binary form as an executable program, or part of
//  an executable program must, when run for the first time by a given
//  user, prominently display the above copyright notice, this list of
//  conditions and the following disclaimer.
//
//  Neither the name of the University of California, Berkeley nor the
//  names of its contributors may be used to endorse or promote products
//  derived from this software without specific prior written
//  permission.
//
// THIS SOFTWARE IS PROVIDED BY THE COPYRIGHT HOLDERS AND CONTRIBUTORS
// "AS IS" AND ANY EXPRESS OR IMPLIED WARRANTIES, INCLUDING, BUT NOT
// LIMITED TO, THE IMPLIED WARRANTIES OF MERCHANTABILITY AND FITNESS FOR
// A PARTICULAR PURPOSE ARE DISCLAIMED.  IN NO EVENT SHALL THE COPYRIGHT
// OWNER OR CONTRIBUTORS BE LIABLE FOR ANY DIRECT, INDIRECT, INCIDENTAL,
// SPECIAL, EXEMPLARY, OR CONSEQUENTIAL DAMAGES (INCLUDING, BUT NOT
// LIMITED TO, PROCUREMENT OF SUBSTITUTE GOODS OR SERVICES; LOSS OF USE,
// DATA, OR PROFITS; OR BUSINESS INTERRUPTION) HOWEVER CAUSED AND ON ANY
// THEORY OF LIABILITY, WHETHER IN CONTRACT, STRICT LIABILITY, OR TORT
// (INCLUDING NEGLIGENCE OR OTHERWISE) ARISING IN ANY WAY OUT OF THE USE
// OF THIS SOFTWARE, EVEN IF ADVISED OF THE POSSIBILITY OF SUCH DAMAGE.
//
//-------------------------------------------------------------------
//-------------------------------------------------------------------

#include <fstream>
#include <iostream>
#include "slVector.H"
#include "slcUtil.H"


double LsSurface::eval(const SlVector3 &x) const {
	SlVector3 y(x-lc);
	int i,j,k;
	i = (int)floor(y[0] / h);
	j = (int)floor(y[1] / h);
	k = (int)floor(y[2] / h);
	double w0, w1, w2;
	w0 = (y[0] - i*h)/h;
	w1 = (y[1] - j*h)/h;
	w2 = (y[2] - k*h)/h;

	return (1.0-w0)*(1.0-w1)*(1.0-w2)*phi(i,j,k)+
		(1.0-w0)*(1.0-w1)*(w2)*phi(i,j,k+1)+
		(1.0-w0)*(w1)*(1.0-w2)*phi(i,j+1,k)+
		(1.0-w0)*(w1)*(w2)*phi(i,j+1,k+1)+
		(w0)*(1.0-w1)*(1.0-w2)*phi(i+1,j,k)+
		(w0)*(1.0-w1)*(w2)*phi(i+1,j,k+1)+
		(w0)*(w1)*(1.0-w2)*phi(i+1,j+1,k)+
		(w0)*(w1)*(w2)*phi(i+1,j+1,k+1);
}


LsSurface::LsSurface(const int &nx, const int &ny, const int &nz, 
										 const double &h, const SlVector &lc, const SlVector &uc) {
	this->nx = nx;
	this->ny = ny;
	this->nz = nz;
	this->h = h;
	this->lc = lc;
	this->uc = uc;
	this->phi.allocate(nx+1,ny+1,nz+1);
}

LsSurface::init(const int &nx, const int &ny, const int &nz, 
								const double &h, const SlVector &lc, const SlVector &uc) {
	this->nx = nx;
	this->ny = ny;
	this->nz = nz;
	this->h = h;
	this->lc = lc;
	this->uc = uc;
	this->phi.allocate(nx+1,ny+1,nz+1);
	return true;
}


//-------------------------------------------------------------------
//-------------------------------------------------------------------
//
// RCS Revision History
//
// $Log: lsSurface.cpp,v $
//
//-------------------------------------------------------------------
//-------------------------------------------------------------------
