//-------------------------------------------------------------------
//-------------------------------------------------------------------
//
// LevelSet Surface Tracker
//   -- Header File
//  
// Primary Author: Adam Bargteil (adamb@cs.utah.edu)
// 
//-------------------------------------------------------------------
//-------------------------------------------------------------------
//
// Copyright (c) 2008-2009, Utah State Board of Regents.  All
// rights reserved.
//
// Redistribution and use in source and binary forms, with or without
// modification, are permitted provided that the following conditions are
// met:
//
//   Redistributions of source code must retain the above copyright
//  notice, this list of conditions and the following disclaimer.
//
//  Redistributions in binary form must reproduce the above copyright
//  notice, this list of conditions and the following disclaimer in the
//  documentation and/or other materials provided with the distribution.
//
//  Redistributions in binary form as an executable program, or part of
//  an executable program must, when run for the first time by a given
//  user, prominently display the above copyright notice, this list of
//  conditions and the following disclaimer.
//
//  Neither the name of the University of California, Berkeley nor the
//  names of its contributors may be used to endorse or promote products
//  derived from this software without specific prior written
//  permission.
//
// THIS SOFTWARE IS PROVIDED BY THE COPYRIGHT HOLDERS AND CONTRIBUTORS
// "AS IS" AND ANY EXPRESS OR IMPLIED WARRANTIES, INCLUDING, BUT NOT
// LIMITED TO, THE IMPLIED WARRANTIES OF MERCHANTABILITY AND FITNESS FOR
// A PARTICULAR PURPOSE ARE DISCLAIMED.  IN NO EVENT SHALL THE COPYRIGHT
// OWNER OR CONTRIBUTORS BE LIABLE FOR ANY DIRECT, INDIRECT, INCIDENTAL,
// SPECIAL, EXEMPLARY, OR CONSEQUENTIAL DAMAGES (INCLUDING, BUT NOT
// LIMITED TO, PROCUREMENT OF SUBSTITUTE GOODS OR SERVICES; LOSS OF USE,
// DATA, OR PROFITS; OR BUSINESS INTERRUPTION) HOWEVER CAUSED AND ON ANY
// THEORY OF LIABILITY, WHETHER IN CONTRACT, STRICT LIABILITY, OR TORT
// (INCLUDING NEGLIGENCE OR OTHERWISE) ARISING IN ANY WAY OUT OF THE USE
// OF THIS SOFTWARE, EVEN IF ADVISED OF THE POSSIBILITY OF SUCH DAMAGE.
//
//-------------------------------------------------------------------
//-------------------------------------------------------------------

#ifndef lsSurface_H
#define lsSurface_H

#include <fstream>
#include <iostream>
#include "slVector.H"
#include "slArray.H"


class LsSurface {
public:
	double eval(const SlVector3 &x) const;
	inline double &operator()(const unsigned int i, const unsigned int j, const unsigned int k);
	inline double &operator()(const unsigned int i, const unsigned int j, const unsigned int k) const;
	LsSurface() {};
	LsSurface(const int &nx, const int &ny, const int &nz, 
						const double &h, const SlVector3 &lc, const SlVector3 &uc);
	bool init(const int &nx, const int &ny, const int &nz, 
						const double &h, const SlVector3 &lc, const SlVector3 &uc);
	SlArray3D<double> phi;
	int nx, ny, nz;
	double h;
	SlVector3 lc, uc;
};

inline double &LsSurface::operator()(const unsigned int i, const unsigned int j, const unsigned int k) {
	return phi(i,j,k);
};

inline double &LsSurface::operator()(const unsigned int i, const unsigned int j, const unsigned int k) const{
	return phi(i,j,k);
};

#endif


//-------------------------------------------------------------------
//-------------------------------------------------------------------
//
// RCS Revision History
//
// $Log: lsSurface.H,v $
//
//-------------------------------------------------------------------
//-------------------------------------------------------------------
