//-------------------------------------------------------------------
//-------------------------------------------------------------------
//
// SlArray
//   -- Header File
//  
// Primary Author: Adam Bargteil (adamb@cs.utah.edu)
//
//-------------------------------------------------------------------
//-------------------------------------------------------------------
//
// Copyright (c) 2008-2009, Utah State Board of Regents.  All
// rights reserved.
//
// Redistribution and use in source and binary forms, with or without
// modification, are permitted provided that the following conditions are
// met:
//
//   Redistributions of source code must retain the above copyright
//  notice, this list of conditions and the following disclaimer.
//
//  Redistributions in binary form must reproduce the above copyright
//  notice, this list of conditions and the following disclaimer in the
//  documentation and/or other materials provided with the distribution.
//
//  Redistributions in binary form as an executable program, or part of
//  an executable program must, when run for the first time by a given
//  user, prominently display the above copyright notice, this list of
//  conditions and the following disclaimer.
//
//  Neither the name of the University of California, Berkeley nor the
//  names of its contributors may be used to endorse or promote products
//  derived from this software without specific prior written
//  permission.
//
// THIS SOFTWARE IS PROVIDED BY THE COPYRIGHT HOLDERS AND CONTRIBUTORS
// "AS IS" AND ANY EXPRESS OR IMPLIED WARRANTIES, INCLUDING, BUT NOT
// LIMITED TO, THE IMPLIED WARRANTIES OF MERCHANTABILITY AND FITNESS FOR
// A PARTICULAR PURPOSE ARE DISCLAIMED.  IN NO EVENT SHALL THE COPYRIGHT
// OWNER OR CONTRIBUTORS BE LIABLE FOR ANY DIRECT, INDIRECT, INCIDENTAL,
// SPECIAL, EXEMPLARY, OR CONSEQUENTIAL DAMAGES (INCLUDING, BUT NOT
// LIMITED TO, PROCUREMENT OF SUBSTITUTE GOODS OR SERVICES; LOSS OF USE,
// DATA, OR PROFITS; OR BUSINESS INTERRUPTION) HOWEVER CAUSED AND ON ANY
// THEORY OF LIABILITY, WHETHER IN CONTRACT, STRICT LIABILITY, OR TORT
// (INCLUDING NEGLIGENCE OR OTHERWISE) ARISING IN ANY WAY OUT OF THE USE
// OF THIS SOFTWARE, EVEN IF ADVISED OF THE POSSIBILITY OF SUCH DAMAGE.
//
//-------------------------------------------------------------------
//-------------------------------------------------------------------

#ifndef __SLARRAY_H__
#define __SLARRAY_H__
#include <iostream>
#include <cassert>
#include "slIO.H"

//template <typename T, bool ByRef>
//struct PassTypeHelper
//{
//	typedef const T& type;
//};
//
//template <typename T>
//struct PassTypeHelper<T,false>
//{
//	typedef T type;
//};
//
//template <typename T>
//struct PassType
//{
//	typedef typename PassTypeHelper<T, sizeof(T) > sizeof(int)>::type;
//};

template <typename T>
class SlArray3D {
	T *data;
	unsigned int _nx, _ny, _nz, nynz;
	//typedef typename PassType<T>::type parameter_type;
public:
	unsigned int N;
	inline T &operator()(unsigned int i, unsigned int j, unsigned int k);
	inline const T &operator()(unsigned int i, unsigned int j, unsigned int k) const;
	inline T& operator[](unsigned int i) { return data[i]; }
	inline const T& operator[](unsigned int i) const { return data[i]; }
	//const T &operator=(parameter_type d);
	const T &operator=(const T& d);
	inline const SlArray3D<T> &operator=(const SlArray3D<T> &d);
	inline SlArray3D<T> &operator-=(const SlArray3D<T>& rhs);
	inline SlArray3D();
	inline SlArray3D(unsigned int nx, unsigned int ny, unsigned int nz);
	inline ~SlArray3D();
	inline bool allocate(unsigned int nx, unsigned int ny, unsigned int nz);
	unsigned int nx() const {return _nx;};
	unsigned int ny() const {return _ny;};
	unsigned int nz() const {return _nz;};

    typedef T*             iterator;
    typedef const T*       const_iterator;

    // iterator support
    iterator begin() { return data; }
    const_iterator begin() const { return data; }
    iterator end() { return data+N; }
    const_iterator end() const { return data+N; }
};

template <typename T> std::istream &operator>>(std::istream &strm, SlArray3D<T> &v);
template <typename T> std::ostream &operator<<(std::ostream &strm, const SlArray3D<T> &v);
template <typename T> T dot(const SlArray3D<T>& a, const SlArray3D<T>& b);

template <typename T>
inline bool SlArray3D<T>::allocate(unsigned int x, unsigned int y, unsigned int z) {
	_nx = x;
	_ny = y;
	_nz = z;
	nynz = _ny*_nz;
	N = _nx*nynz;
	data = new T[N];
	return true;
}

template<class T>
inline SlArray3D<T> operator*(const SlArray3D<T> &d, T x) {
	SlArray3D<T> y(d.nx(), d.ny(), d.nz());
	for (unsigned int i=0; i<d.nx(); i++)
		for (unsigned int j=0; j<d.ny(); j++)
			for (unsigned int k=0; k<d.nz(); k++)
				y(i,j,k) = d(i,j,k)*x;
	return y;
}

template <class T>
/*inline*/ const T &SlArray3D<T>::operator=(const T &x) {
	T *d = data;
//	for (unsigned int i=0; i<_nx; i++) {
//		for (unsigned int j=0; j<_ny; j++) {
//			for (unsigned int k=0; k<_nz; k++, d++) {
//				(*d) = x;
//			}
//		}
//	}
	for (unsigned int i=0; i < N; ++i, ++d)  {
		(*d) = x;
	}
	return x;
}


template <class T>
inline const SlArray3D<T> &SlArray3D<T>::operator=(const SlArray3D<T> &x) {
	assert(N == x.N && _nx == x._nx && _ny == x._ny && _nz == x._nz && "Invalid assignment: Different array sizes");
	if( this != &x ) {
		T *d = data;
		T *y = x.data;
	//	for (unsigned int i=0; i<_nx; i++) {
	//		for (unsigned int j=0; j<_ny; j++) {
	//			for (unsigned int k=0; k<_nz; k++, d++, y++) {
	//				(*d) = (*y);
	//			}
	//		}
	//	}
		for(unsigned int i=0; i < N; ++i, ++d, ++y) {
			(*d) = (*y);
		}
	}
	return *this;
}

template <class T>
inline SlArray3D<T> &SlArray3D<T>::operator-=(const SlArray3D<T>& rhs) {
	assert(N == rhs.N && _nx == rhs._nx && _ny == rhs._ny && _nz == rhs._nz && "Invalid operator: Different array sizes");
	T* d = data;
	T *y = rhs.data;
	for(unsigned int i=0; i < N; ++i, ++d, ++y) {
		(*d) -= (*y);
	}
	return *this;
}

template<class T>
T dot(const SlArray3D<T>& a, const SlArray3D<T>& b) {
	assert(a.N == b.N && "Invalid operator: Different array sizes");
	T sum = 0.0;
	for( unsigned int i=0; i < a.N; ++i ) {
		sum += a[i] * b[i];
	}
	return sum;
}

template <class T>
inline T &SlArray3D<T>::operator()(unsigned int i, unsigned int j, unsigned int k) {
	assert( i*nynz+j*_nz+k < N );
	return data[i*nynz+j*_nz+k]; // data[(i*_ny + j)*_nz + k]
}

template <class T>
inline const T &SlArray3D<T>::operator()(unsigned int i, unsigned int j, unsigned int k) const {
	assert( i*nynz+j*_nz+k < N );
	return data[i*nynz+j*_nz+k];
}

template <class T>
inline SlArray3D<T>::SlArray3D():data(NULL), _nx(0), _ny(0), _nz(0), nynz(0), N(0) {}

template <class T>
inline SlArray3D<T>::SlArray3D(unsigned int x, unsigned int y, unsigned int z): _nx(x), _ny(y), _nz(z), nynz(y*z), N(x*y*z) {
	data = new T[N];
}

template <class T>
inline SlArray3D<T>::~SlArray3D() {
	if (data) delete [] data;
}



#if 1
template <class T>
std::istream &operator>>(std::istream &strm, SlArray3D<T> &v) {
	unsigned int nx,ny,nz,i,j,k;
	std::ios::fmtflags orgFlags = strm.setf(std::ios::skipws);

	eatStr("[", strm);
	strm >> nx;
	eatStr(",", strm);
	strm >> ny;
	eatStr(",", strm);
	strm >> nz;
	eatStr("]", strm);
	
	eatStr("[", strm);
	v(nx, ny, nz);
	for (i=0; i<nx; i++) {
		for (j=0; j<ny; j++) {
			for (k=0; k<nz; k++) {
				strm >> v(i,j,k);
			}
		}
	}
	eatStr("]", strm);
	strm.flags(orgFlags);
	return strm;
}

template <typename T>
std::ostream &operator<<(std::ostream &strm, const SlArray3D<T> &v) {
	strm << "[";
	strm << v.nx() << "," << v.ny() << "," <<v.nz() << "]";
	unsigned int i, j, k;
	strm<<"["<<std::endl;
	for (i=0; i<v.nx(); i++) {
		for (j=0; j<v.ny(); j++) {
			for (k=0; k<v.nz(); k++) {
				strm << v(i,j,k) << " ";
			}
		}
	}
	strm<<"]\n";
	return strm;
}
#endif
#endif

